/*
 * Copyright (c) 2011 Michael Pippig
 *
 * This file is part of PNFFT.
 *
 * PNFFT is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * PNFFT is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with PNFFT.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#include "pnfft.h"
#include "ipnfft.h"

static int local_nodes_out_of_range(
    const R *lo, const R *up, const R *x_max);
static R random_number_less_than_one(
    void);
static void default_fft_size(
    const INT *N,
    INT *n, R *x_max);
static int default_m(
    void);


void PNX(init_x_3d_adv)(
    const R *lo, const R *up,
    const R *x_max, INT loc_M,
    R *x
    )
{
  R tmp;
  
  if(local_nodes_out_of_range(lo, up, x_max))
    return;
  
  for (INT j=0; j<loc_M; j++){
    for(int t=0; t<3; t++){
      do{
        tmp = random_number_less_than_one();
        tmp = (up[t]-lo[t]) * tmp + lo[t];
      }
      while( (-x_max[t] < tmp) || (tmp <= x_max[t]) );
      x[3*j+t] = tmp;
    }
  }
}


static int local_nodes_out_of_range(
    const R *lo, const R *up, const R *x_max
    )
{
  int rtnval = 0;
  
  for(int t=0; t<3; t++)
    if( (lo[t] < -x_max[t]) || (x_max[t] < up[t]) )
      rtnval = 1;
  
  return rtnval;
}


static R random_number_less_than_one(
    void
    )
{
  int itmp;
  R tmp;
  
  do {
    itmp = rand();
    tmp = (R) itmp/RAND_MAX;
  } while(tmp==1.0);
  
  return tmp;
}
    
    

void PNX(local_size_adv)(
    int d, const INT *N, MPI_Comm comm_cart,
    INT *local_N, INT *local_N_start,
    R *lower_border, R *upper_border
    )
{
  int m;
  INT n[3];
  R x_max[3];

  m = default_m();
  default_fft_size(N,
      n, x_max);

  PNX(local_size_guru)(
      d, N, n, x_max, m, comm_cart,
      local_N, local_N_start, lower_border, upper_border);
}


void PNX(init_adv)(
    PNX(plan) *ths, int d, const INT *N,
    INT local_M,
    unsigned nfft_flags, unsigned fftw_flags,
    MPI_Comm comm_cart
    )
{
  int m;
  INT n[3];
  R x_max[3];

  m = default_m();
  default_fft_size(N,
      n, x_max);

  PNX(init_guru)(
      ths, d, N, n, x_max, local_M, m,
      nfft_flags, fftw_flags, comm_cart);
}


static void default_fft_size(
    const INT *N,
    INT *n, R *x_max
    )
{
  for(int t=0; t<3; t++){
    n[t] = 2*N[t];
    x_max[t] = 0.5;
  }
}


static int default_m(
    void
    )
{
  return 6;
}
